# -*- encoding: utf-8 -*-


from django.test import TestCase
from django.db import models
from models import WorkflowMixin


#codes actions
ACTION_0 = 'action_0'
ACTION_1 = 'action_1'
ACTION_2 = 'action_2'
ACTION_3 = 'action_3'
ACTION_4 = 'action_4'
ACTION_5 = 'action_5'

# codes états
ETAT_0 = 'etat_0'
ETAT_1 = 'etat_1'
ETAT_2 = 'etat_2'
ETAT_3 = 'etat_3'
ETAT_4 = 'etat_4'

#libellés états
ETATS = {
    ETAT_0 : ETAT_0,
    ETAT_1 : ETAT_1,
    ETAT_2 : ETAT_2,
    ETAT_3 : ETAT_3,
    ETAT_4 : ETAT_4,
    }

# définition du worflow séquentiel
ACTIONS = {
    ACTION_0 : {
        'nom' : ACTION_0,
        'etat_initial' : None,
        'etat_final' : ETAT_0,
    },
    ACTION_1 : {
        'nom' : ACTION_1,
        'etat_initial' : ETAT_0,
        'etat_final' : ETAT_1,
    },
    ACTION_2 : {
        'nom' : ACTION_2,
        'etat_initial' : ETAT_1,
        'etat_final' : ETAT_2,
    },
    ACTION_3 : {
        'nom' : ACTION_3,
        'etat_initial' : ETAT_2,
        'etat_final' : ETAT_3,
    },
    ACTION_4 : {
        'nom' : ACTION_4,
        'etat_initial' : ETAT_0,
        'etat_final' : ETAT_3,
    },
    ACTION_5 : {
        'nom' : ACTION_5,
        'etat_initial' : (ETAT_0, ETAT_1,),
        'etat_final' : ETAT_4,
    },
}

class KlassWorkflow(WorkflowMixin):
    """
    Classe décrivant un workflow
    """
    etat_initial = ETAT_0
    etats = ETATS
    actions = ACTIONS
    passage = None

    def acces_action_0(self, request, action):
        return True

    def acces_action_1(self, request, action):
        return True

    def acces_action_2(self, request, action):
        return True

    def acces_action_3(self, request, action):
        return True

    def acces_action_4(self, request, action):
        return True

    def acces_action_5(self, request, action):
        return True

    def passage_de_none_a_etat_0(self, request, action):
        self.passage = 0

    def passage_de_etat_0_a_etat_1(self, request, action):
        self.passage = 1

    def passage_de_etat_0_a_etat_0(self, request, action):
        self.passage = 2

    class Meta:
        abstract = True


class WFKlass(KlassWorkflow, models.Model):
    """
    Modèle django classique héritant d'un classe de workflow
    """
    titre = models.CharField(max_length=10,)

    def __unicode__(self, ):
        return self.titre

class WorkFlowTest(TestCase):

    def test_passage_initial(self):
        """
        test de l'initialisation de l'état initial
        """
        obj = WFKlass(titre="1")
        obj.save()
        self.assertEqual(obj.etat, ETAT_0)
        self.assertEqual(obj.passage, 0)

    def test_passage_etat(self):
        """
        test d'un passage d'état permis
        """
        obj = WFKlass(titre="2")
        obj.save()
        self.assertEqual(obj.etat, ETAT_0)
        self.assertEqual(obj.passage, 0)
        obj.etat = ETAT_1
        obj.save()
        self.assertEqual(obj.etat, ETAT_1)
        self.assertEqual(obj.passage, 1)

    def test_passage_interdit(self):
        """
        WF impossible de 0 à 2
        """
        obj = WFKlass(titre="3")
        obj.save()
        self.assertRaises(Exception, setattr, obj, 'etat', ETAT_2)

    def test_fixer_au_meme_etat(self):
        """
        test que le fonction de sauvegarde au même état est appelée
        """
        obj = WFKlass(titre="4")
        obj.save()
        self.assertEqual(obj.etat, ETAT_0)
        obj.save()
        self.assertEqual(obj.etat, ETAT_0)
        self.assertEqual(obj.passage, 2)

    def test_etats_initiaux_multiples(self):
        """
        test avec provenances multiples pour une action
        """
        obj = WFKlass(titre="5")
        obj.save()
        self.assertEqual(obj.etat, ETAT_0)
        obj.etat = ETAT_4
        obj.save()
        self.assertEqual(obj.etat, ETAT_4)

        obj = WFKlass(titre="6")
        obj.save()
        obj.etat = ETAT_1
        self.assertEqual(obj.etat, ETAT_1)
        obj.etat = ETAT_4
        obj.save()
        self.assertEqual(obj.etat, ETAT_4)

    def test_etats_initiaux_validation(self):
        """
        test si l'action à états initiaux multiples conserve la validation
        """
        obj = WFKlass(titre="7")
        obj.etat = ETAT_1
        obj.etat = ETAT_2
        self.assertRaises(Exception, setattr, obj, 'etat', ETAT_4)

    def test_actions_a_la_volee(self):
        """
        test si la coherence des etats est respectée sans sauvegarder l'objet
        """
        obj = WFKlass(titre="8")
        obj.etat = ETAT_1
        obj.etat = ETAT_2
        self.assertRaises(Exception, setattr, obj, 'etat', ETAT_1)

    def test_etat_dispo_liste(self):
        """
        test si la liste d'états possible reflète le workflow défini
        """
        obj = WFKlass(titre="9")
        self.assertEqual(obj.etat, ETAT_0)
        etats_dispo = [k for k, v in obj.get_etats_disponibles()].sort()
        etats_attendus = [ETAT_0, ETAT_1, ETAT_3, ETAT_4].sort()
        self.assertEqual(etats_dispo, etats_attendus)
